﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;

namespace linqFirstTry
{
    // Bart De Smet: C# 5.0 unleashed, page 922
    public class Program
    {
        private static readonly IList<Product> Products = new List<Product>
        {
            // De Smets, page 921
            new Product {Name = "Chai", Price = 18.00m, CategoryId = 1},
            // m, means decimal!!
            new Product {Name = "Chang", Price = 19.00m, CategoryId = 2},
            new Product {Name = "Anissed Suryp", Price = 10.00m, CategoryId = 1},
            new Product {Name = "Icecream", Price = 7.00m, CategoryId = 1},
            new Product {Name = "Another Icecream", Price = 7.00m}
        };

        private static readonly IList<Category> Categories = new List<Category>
        {
            new Category {Id=1, Name="Beverages"},
            new Category {Id=2, Name="Other"}
        };

        static void Main()
        {
            //AllProducts();
            //CheapProducts(15.0m);
            //CheapProductsNonLinq(15.0m);
            //QuerySyntaxExamples();
            //Ordering();
            //Joining();
            JoiningMemoryUsers();
        }

        public static void AllProducts()
        {
            var allProducts = from product in Products
                              where product.Price > 0
                              select product;
            Console.WriteLine(String.Join("\n", allProducts));
        }

        private static void CheapProducts(decimal upperPrice)
        {
            var cheapProducts = from product in Products
                                where product.Price <= upperPrice
                                //orderby product.Price, product.Name
                                select product;
            //limit = limit*2;
            Console.WriteLine(String.Join("\n", cheapProducts));
        }

        private static void CheapProductsNonLinq(decimal upperPrice)
        {
            IList<Product> cheapProducts = new List<Product>();
            foreach (var product in Products)
            {
                if (product.Price <= upperPrice)
                {
                    cheapProducts.Add(product);
                }
            }
            Console.WriteLine(String.Join("\n", cheapProducts));
        }

        private static void SelectExample()
        {
            var cheap = from product in Products
                        where product.Price < 25m
                        select product.Name; // selecting a single property

            var cheap2 = from product in Products
                         where product.Price < 25m
                         select new { product.Name, product.Price }; // anonymous type
        }

        private static void QuerySyntaxExamples()
        {
            // ordinary LINQ syntax
            var names1 = from product in Products
                         where product.Price < 15
                         select product.Name;
            Console.WriteLine(String.Join(", ", names1));

            // LINQ expression syntax
            var names2 = Products
                .Where(product => product.Price < 15)
                .Select(product => product.Name);
            Console.WriteLine(String.Join(", ", names2));
        }

        private static int FindSmallestNonLinq(int[] data)
        {
            int smallestSoFar = int.MinValue;
            for (int i = 0; i < data.Length; i++)
            {
                if (data[i] < smallestSoFar)
                {
                    smallestSoFar = data[i];
                }
            }
            return smallestSoFar;
        }

        private static int FindSmallestLinq(IEnumerable<int> data)
        {
            var smallest = data.Min();
            return smallest;
        }

        // inner query
        private static IEnumerable<Product> CheapestProducts()
        {
            var cheapest = from product in Products
                           where product.Price == Products.Min(product2 => product2.Price)
                           select product;
            return cheapest;
        }

        private static void Ordering()
        {
            var prod = from product in Products
                       orderby product.Price
                       select product;
            Console.WriteLine(String.Join(", ", prod));
        }

        private static void Joining()
        {
            var prod = from product in Products
                       join category in Categories
                           on product.CategoryId equals category.Id
                       select new { Name = product.Name, Category = category.Name };
            Console.WriteLine(String.Join(", ", prod));
        }

        private static void JoiningMemoryUsers()
        {
            // De Smets page 962
            const int MB = 1024 * 1024;
            var heavyMemoryUsers = (from process in Process.GetProcesses()
                                    where process.WorkingSet64 > 100 * MB
                                    orderby process.WorkingSet64 descending
                                    select new
                                    {
                                        Name = process.ProcessName,
                                        Memory = process.WorkingSet64
                                    }).Take(10);
            //foreach (var heavyMemoryUser in heavyMemoryUsers)
            //{
            //    Console.WriteLine(heavyMemoryUser);
            //}
            Console.WriteLine(String.Join("\n", heavyMemoryUsers));
        }
    }
}
